<?php
/**
 * @file
 * Non-hook functions for civicrm_private_report module.
 */

/**
 * Records whether a given report instance is or is not a user-private report.
 *
 * @param $instance_id
 *  The relevant CiviCRM report instance ID.
 * @param $is_private
 *  Boolean: is private, or is not.
 *
 * @return Boolean: return value of the db_query call that records the setting.
 */
function _civicrm_private_report_set_private_report($instance_id, $is_private) {
  if ($is_private) {
    global $user;
    $data = array(
      'instance_id' => $instance_id,
      'uid'         => $user->uid,
    );
    return (bool)drupal_write_record('civicrm_private_report', $data);
  }
  else {
    $query = "DELETE FROM {civicrm_private_report} WHERE instance_id = %d";
    return (bool)db_query($query, $instance_id);
  }
}

/**
 * Checks wither a given report is private, and whether it belongs to the
 * given user, if any.
 * @param $instance_id
 *  The relevant CiviCRM report instance ID.
 * @param $uid
 *  Optional. If given, the uid of the user whose ownership we're testing.
 *
 * @return Boolean
 *  When only $intance-id is given: TRUE if the report instance is a user-private
 *  report, or FALSE if not.
 *  When the optional $uid is also given: TRUE if the report is private and
 *  owned by that user, or FALSE otherwise.
 */
function _civicrm_private_report_is_private($instance_id, $uid = NULL) {
  $query = "SELECT 1 FROM {civicrm_private_report} WHERE instance_id = %d";
  if (isset($uid)) {
    $query .= " AND uid = %d";
  }
  return (bool)db_result(db_query($query, $instance_id, $uid));
}

/**
 * Gets data from civicrm_private_report table.
 * @param $select
 *  A single-dimensional array of columns to select. If empty, all columns
 *  will be selected.
 * @param $where
 *  Optional. An assocative array of column_name => value pairs to use in a
 *  WHERE clause.
 * @param $order
 *  Optional. Array of ORDER BY settings. Each item in the array can be a simple
 *  value, which will be converted to an ASCENDING sort; or it can be a
 *  string-keyed item, in which the key is the column name, and the value is
 *  either "ASC" or "DESC".
 * @param $single_column
 *  Boolean. If TRUE, values will be returned as a single-dimensional array. If
 *  FALSE (the default), values will be returned as a numerically-indexed array
 *  of associative arrays of column_name => value pairs.
 *
 * @return Array
 *  The matching rows. See $single_column parameter.
 */
function _civicrm_private_report_get_rows($select = array(), $where = array(), $order = array(), $single_column = FALSE) {
  $select_clause = $where_clause = $order_by_clause = '';
  $params = array();

  // Define valid columns.
  $valid_columns = array(
    'id',
    'instance_id',
    'uid',
  );
  // Ensure $select contains only valid columns.
  if (is_array($select) && !empty($select)) {
    $select = array_intersect($select, $valid_columns);
  }
  else {
    $select = $valid_columns;
  }
  $select_clause = 'SELECT '. implode(',', $select);

  // Set up WHERE clause.
  if (is_array($where) && count($where)) {
    $wheres = $params = array();
    foreach ($valid_columns as $column) {
      if (array_key_exists($column, $where)) {
        $wheres[] = "$column = %d";
        $params[] = $where[$column];
      }
    }
    if (!empty($wheres)) {
      $where_clause = 'WHERE '. implode(' AND ', $wheres);
    }
  }

  // Set up ORDER BY clause.
  if (is_array($order) && !empty($order)) {
    $orderbys = array();
    foreach ($order as $column => $direction) {
      if (drupal_strtolower($direction) != 'desc' && drupal_strtolower($direction) != 'asc') {
        $column = $direction;
        $direction = 'ASC';
      }
      if (in_array($column, $valid_columns)) {
        $orderbys[] = "$column $direction";
      }
    }
    $order = array_intersect($order, $valid_columns);
    $order_by_clause = 'ORDER BY '. implode(', ', $orderbys);
  }

  // Build and execute the query.
  $query = "$select_clause FROM {civicrm_private_report} $where_clause $order_by_clause";
  $result = db_query($query, $params);
  $rows = array();
  while ($row = db_fetch_array($result)) {
    if ($single_column) {
      $rows[] = $row[$select[0]];
    }
    else {
      $rows[] = $row;
    }
  }
  return $rows;
}

/**
 * Gets a list of distinct uid => name pairs for users having user-private reports.
 *
 * @param $uid
 *  Optional. If given, the list is limited to the given user ID. (Note that
 *  this can result in an empty array being returned, if that user has no
 *  user-private reports.)
 *
 * @return Array of uid => name pairs.
 */
function _civicrm_private_report_get_users($uid = 0) {
  $users = array();
  $query = '
    SELECT DISTINCT u.name, u.uid
    FROM {users} u
      INNER JOIN {civicrm_private_report} pr ON u.uid = pr.uid
  ';
  if ($uid) {
    $query .= ' WHERE u.uid = %d ';
  }
  $query .= ' ORDER BY name ';
  $result = db_query($query, $uid);
  while ($row = db_fetch_array($result)) {
    $users[$row['uid']] = $row['name'];
  }
  return $users;
}

/**
 * Gets the page output for menu item "civicrm_private_report/menu/list/all".
 */
function _civicrm_private_report_menu_list_all() {
  $output = drupal_get_form('_civicrm_private_report_form_list_filter');
  $output .= _civicrm_private_report_build_list_all();
  return $output;
}

/**
 * Gets the page output for menu item "civicrm_private_report/menu/list".
 */
function _civicrm_private_report_menu_list() {
  $output = '<p>'. t('This page lists all CiviCRM user-private reports currently available for you.') .'</p>';
  global $user;
  $output .= '<div id="crm-container">'. _civicrm_private_report_user_list($user->uid) .'</div>';
  return $output;
}

/**
 * Form constructor for the user filter form.
 *
 * @see _civicrm_private_report_form_list_filter_validate()
 * @ingroup forms
 */
function _civicrm_private_report_form_list_filter($form_state) {
  $form =array();

  // Build list of users having private reports.
  $options = array('0' => '- '. t('Show All') .' -');
  $options += _civicrm_private_report_get_users();

  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter users'),
  );
  $form['filters']['#theme'] = 'civicrm_private_report_user_filter';
  $form['filters']['uid'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => (isset($_SESSION['civicrm_private_report_list_filter']) ? $_SESSION['civicrm_private_report_list_filter'] : 0),
  );
  $form['filters']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('List'),
  );
  return $form;
}

/**
 * Form submission handler for the user filter form.
 */
function _civicrm_private_report_form_list_filter_submit($form, &$form_state) {
  $filter = &$_SESSION['civicrm_private_report_list_filter'];
  $filter = $form_state['values']['uid'];
}

/**
 * Gets page output for listing the user-private reports of one given user.
 */
function _civicrm_private_report_user_list($uid) {
  global $user;
  $private_instance_ids = _civicrm_private_report_get_rows(array('instance_id'), array('uid' => $uid), array(), TRUE);

  // Build a list of private reports with relevant info for the template listing.
  _civicrm_private_report_civicrm_html_head();
  if (!empty($private_instance_ids)) {
    require_once 'CRM/Report/Page/InstanceList.php';
    require_once 'CRM/Core/Smarty.php';

    $private_list = array();
    $info = CRM_Report_Page_InstanceList::info();
    foreach ($info as $category => $instances) {
      $category_private = array();
      foreach ($instances as $instance_id => $instance) {
        if (in_array($instance_id, $private_instance_ids)) {
          // If current user is report owner, alter delete URL to point to this
          // module's delete path.
          if ($user->uid == $uid) {
            $instance['deleteUrl'] = url('civicrm_private_report/delete/'. $instance_id);
          }
          // If current user is NOT report owner, hide the delete URL entirely.
          else {
            unset($instance['deleteUrl']);
          }
          // Add the instance to the list of private reports.
          $category_private[$instance_id] = $instance;
        }
      }
      if (!empty($category_private)) {
        $private_list[$category] = $category_private;
      }
    }
  }

  // If we actually have anything in the private reports list, parse the template and display output.
  $output = '';
  if (!empty($private_list)) {
    $tpl = CRM_Core_Smarty::singleton();
    $tpl->assign('list', $private_list);
    $tpl_output = $tpl->fetch('CRM/Report/Page/InstanceList.tpl');

    $output = $tpl_output;
  }
  else {
    if (user_access('access CiviReport')) {
      $args = array(
        '!link' => l(t('view the list of all CiviCRM Reports'), 'civicrm/report/list', array('query' => 'reset=1')),
      );
      $view_reports = ' '. t('You can !link.', $args);
    }
    $output = '<p>'. t('No user-private reports were found for you.') . $view_reports .'</p>';
  }
  return $output;
}

/**
 * Menu page callback for path "civicrm_private_report/delete". Attempts to
 * delete the given report and gives feedback to the user.
 */
function _civicrm_private_report_menu_delete($instance_id) {
  if (_civicrm_private_report_delete($instance_id)) {
    drupal_set_message(t('The selected report instance has been deleted.'));
  }
  else {
    drupal_set_message(t('Some problems were encountered in deleting the report. Please review this list to ensure it was deleted.', 'error'));
  }
  drupal_goto('civicrm_private_report/list');
}

/**
 * Deletes the given report instance, both in CiviCRM and as a user-private report.
 *
 * @param $instance_id
 *  The relevant CiviCRM report instance ID.
 *
 * @return Boolean: TRUE on success, FALSE on error.
 */
function _civicrm_private_report_delete($instance_id) {

  // Delete the CiviCRM report.
  civicrm_initialize();
  require_once 'CRM/Report/DAO/Instance.php';
  $dao = new CRM_Report_DAO_Instance();
  $dao->id = $instance_id;
  $result = $dao->delete();
  if ($result === FALSE) {
    return FALSE;
  }

  // Delete the private report record
  $result = _civicrm_private_report_set_private_report($instance_id, FALSE);
  if ($result === FALSE) {
    return FALSE;
  }

  return TRUE;
}

/**
 * Menu access callback for path "civicrm_private_report/delete".
 */
function _civicrm_private_report_menu_access_delete($instance_id) {
  global $user;
  if (!user_access('administer own CiviCRM reports') || !_civicrm_private_report_is_private($instance_id, $user->uid)) {
    return FALSE;
  }
  return TRUE;
}

/**
 * Menu access callback for path "civicrm_private_report/list"
 */
function _civicrm_private_report_menu_access_list() {
  return (user_access('administer own CiviCRM reports') && user_access('access Report Criteria'));
}

/**
 * Sets up JavaScript and CSS for CiviCRM-related content.
 * Essentially copied from civicrm_html_head() in civicrm.module, with the
 * exception that this function doesn't work only under the civicrm/ path.
 *
 * @return a style tag that indicates what file browsers should import
 */
function _civicrm_private_report_civicrm_html_head() {

  if (! civicrm_initialize()) {
    return;
  }

  $head = NULL;

  require_once 'CRM/Core/Config.php';
  $config = CRM_Core_Config::singleton();

  $template =& CRM_Core_Smarty::singleton();
  $buffer = $template->fetch('CRM/common/jquery.files.tpl');
  $lines  = preg_split('/\s+/', $buffer);
  foreach ($lines as $line) {
    $line = trim($line);
    if (empty($line)) {
      continue;
    }
    if (strpos($line, '.js') !== FALSE) {
      drupal_add_js(drupal_get_path('module', 'civicrm') . '/../' . $line);
    }
    elseif (strpos($line, '.css') !== FALSE) {
      drupal_add_css(drupal_get_path('module', 'civicrm') . '/../' . $line);
    }
  }

  // add localized calendar js
  $localisation   = explode('_', $config->lcMessages);
  $localization_file = drupal_get_path('module', 'civicrm') . '/../packages/jquery/jquery-ui-1.8.11/development-bundle/ui/i18n/jquery.ui.datepicker-'. $localisation[0] . '.js';

  if (file_exists($localization_file)) {
    drupal_add_js($localization_file);
  }

  // add Common.js
  drupal_add_js(drupal_get_path('module', 'civicrm') . '/../js/Common.js');

  // add the final assignment
  drupal_add_js('var cj = jQuery.noConflict(); $ = cj;', 'inline');

  if (isset($config->customCSSURL) && ! empty($config->customCSSURL)) {
    $head  = "<style type=\"text/css\">@import url({$config->customCSSURL});</style>\n";
  }
  else {
    drupal_add_css(drupal_get_path('module', 'civicrm') . '/../css/deprecate.css');
    drupal_add_css(drupal_get_path('module', 'civicrm') . '/../css/civicrm.css');
    drupal_add_css(drupal_get_path('module', 'civicrm') . '/../css/extras.css');
  }
  return $head;
}

/**
 * Gets page content for the list of all users' user-private reports, based on
 * the filter settings defined in the user filter form.
 */
function _civicrm_private_report_build_list_all() {
  $filter = &$_SESSION['civicrm_private_report_list_filter'];
  $filter = isset($filter) ? $filter : 0;

  $users = _civicrm_private_report_get_users($filter);

  $output = '<div id="crm-container">';

  foreach ($users as $uid => $name) {
    $output .= '<h3>' . check_plain($name) . '</h3>';
    $output .= _civicrm_private_report_user_list($uid);
  }

  $output .'</div>';
  return $output;
}

/**
 * Returns HTML for the user filter form.
 *
 * @param $form
 *  The user filter form.
 *
 * @ingroup themeable
 */
function theme_civicrm_private_report_user_filter($form) {
  $output = '';
  $output .= '<dl class="multiselect">';
  $output .= '<dt><label>' . t('Select a user') . '</label></dt>';
  $output .= '<dd>' . drupal_render($form['uid']) . '</dd>';
  $output .= '<div class="container-inline">'. drupal_render($form['submit']) .'</div>';
  $output .= '</dl>';
  return $output;
}
